//*** include ***
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <string.h>
#include <sys/param.h>
#include "sdkconfig.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/queue.h"
#include "freertos/event_groups.h"
#include "esp_system.h"
#include "esp_wifi.h"
#include "esp_event.h"
#include "esp_netif.h"
#include "esp_log.h"
#include "spi_flash_mmap.h"
#include "nvs_flash.h"
#include "nvs.h"

#include "lwip/err.h"
#include "lwip/sys.h"
#include "lwip/sockets.h"
#include "lwip/dns.h"
#include "lwip/netdb.h"
#include "mqtt_client.h"

#include "esp_bt.h"
#include "esp_bt_device.h"
#include "esp_gap_ble_api.h"
#include "esp_gattc_api.h"
#include "esp_gatt_defs.h"
#include "esp_bt_main.h"
#include "esp_gatt_common_api.h"

#include <esp_http_server.h>
#include <esp_http_client.h>
#include "esp_ota_ops.h"
#include "esp_sntp.h"
#include "esp_timer.h"
#include "driver/gptimer.h"
#include "driver/ledc.h"
#include "mbedtls/aes.h"
#include "mbedtls/ccm.h"
#include "esp_tls.h"
#include "esp_crt_bundle.h"
#include "phy_init_data.h"
#include "soc/efuse_reg.h"
#include "driver/rmt.h"
#include "driver/i2c.h"

//5.0
#include "hal/gpio_hal.h"
#include "esp_mac.h"
#include "esp_chip_info.h"

//*** define ***
//*** common ***
#define AP_TAG "R4S"

//*** wifi ***
#define WIFI_MAXIMUM_RETRY  11
#define WIFI_CONNECTED_BIT BIT0
#define WIFI_FAIL_BIT      BIT1


//*** bt ***
//redmond
// REMOTE_SERVICE_UUID "6e400001-b5a3-f393-e0a9-e50e24dcca9e"
#define REMOTE_SERVICE_UUID {0x9e, 0xca, 0xdc, 0x24, 0x0e, 0xe5, 0xa9, 0xe0, 0x93, 0xf3, 0xa3, 0xb5, 0x01, 0x00, 0x40, 0x6e}
// REMOTE_TXCHAR_UUID "6e400002-b5a3-f393-e0a9-e50e24dcca9e"
#define REMOTE_TXCHAR_UUID {0x9e, 0xca, 0xdc, 0x24, 0x0e, 0xe5, 0xa9, 0xe0, 0x93, 0xf3, 0xa3, 0xb5, 0x02, 0x00, 0x40, 0x6e}
// REMOTE_RXCHAR_UUID "6e400003-b5a3-f393-e0a9-e50e24dcca9e"
#define REMOTE_RXCHAR_UUID {0x9e, 0xca, 0xdc, 0x24, 0x0e, 0xe5, 0xa9, 0xe0, 0x93, 0xf3, 0xa3, 0xb5, 0x03, 0x00, 0x40, 0x6e}
//xiaomi
//01344736-0000-1000-8000-262837236156
#define XREMOTE_SERVICE_UUID {0x56, 0x61, 0x23, 0x37, 0x28, 0x26, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x36, 0x47, 0x34, 0x01}
//0000fe95-0000-1000-8000-00805f9b34fb
#define XREMOTE_SERVICE1_UUID {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x95, 0xfe, 0x00, 0x00}
#define XREMOTE_SERVICE116_UUID 0xfe95
//0000180a-0000-1000-8000-00805f9b34fb
#define XREMOTE_SERVICE2_UUID {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x0a, 0x18, 0x00, 0x00}
#define XREMOTE_SERVICE216_UUID 0x180a
//0000aa02-0000-1000-8000-00805f9b34fb
#define XREMOTE_STATUS_UUID  {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x02, 0xaa, 0x00, 0x00}
//00000010-0000-1000-8000-00805f9b34fb
#define XREMOTE_AUTHINIT_UUID  {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00}
//00000001-0000-1000-8000-00805f9b34fb
#define XREMOTE_AUTH_UUID  {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00}
//00000004-0000-1000-8000-00805f9b34fb
#define XREMOTE_VER_UUID  {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00}
//0000aa01-0000-1000-8000-00805f9b34fb
#define XREMOTE_SETUP_UUID  {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x01, 0xaa, 0x00, 0x00}
//0000aa04-0000-1000-8000-00805f9b34fb
#define XREMOTE_TIME_UUID  {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x04, 0xaa, 0x00, 0x00}
//0000aa05-0000-1000-8000-00805f9b34fb
#define XREMOTE_BOIL_UUID  {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x05, 0xaa, 0x00, 0x00}
//00002a28-0000-1000-8000-00805f9b34fb
#define XREMOTE_MCUVER_UUID  {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x28, 0x2a, 0x00, 0x00}
// update

//00001800-0000-1000-8000-00805f9b34fb
//#define XREMOTE_SERVICE3_UUID {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x00, 0x18, 0x00, 0x00}
//#define XREMOTE_SERVICE316_UUID 0x1800
//00002a00-0000-1000-8000-00805f9b34fb
//#define XREMOTE_UPDATE_UUID  {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x14, 0x000, 0x00, 0x00}


//0000fee8-0000-1000-8000-00805f9b34fb
#define XREMOTE_SERVICE3_UUID {0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0xe8, 0xfe, 0x00, 0x00}
#define XREMOTE_SERVICE316_UUID 0xfee8
//013784cf-f7e3-55b4-6c4c-9fd140100a16
#define XREMOTE_UPDATE_UUID  {0x16, 0x0a, 0x10, 0x40, 0xd1, 0x9f, 0x4c, 0x6c, 0xb4, 0x55, 0xe3, 0xf7, 0xcf, 0x84, 0x37, 0x01}

//galcon
// GLREMOTE_SERVICE_UUID  "e8680100-9c4b-11e4-b5f7-0002a5d5c51b"
#define GLREMOTE_SERVICE_UUID  {0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0xf7, 0xb5, 0xe4, 0x11, 0x4b, 0x9c, 0x00, 0x01, 0x68, 0xe8}
// GLREMOTE_SERVICE1_UUID "e8680200-9c4b-11e4-b5f7-0002a5d5c51b"
#define GLREMOTE_SERVICE1_UUID {0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0xf7, 0xb5, 0xe4, 0x11, 0x4b, 0x9c, 0x00, 0x02, 0x68, 0xe8}
// GLREMOTE_SERVICE2_UUID "e8680400-9c4b-11e4-b5f7-0002a5d5c51b"
#define GLREMOTE_SERVICE2_UUID {0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0xf7, 0xb5, 0xe4, 0x11, 0x4b, 0x9c, 0x00, 0x04, 0x68, 0xe8}
// GLREMOTE_SETUP_UUID     "e8680401-9c4b-11e4-b5f7-0002a5d5c51b"
#define GLREMOTE_SETUP_UUID    {0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0xf7, 0xb5, 0xe4, 0x11, 0x4b, 0x9c, 0x01, 0x04, 0x68, 0xe8}
// GLREMOTE_AUTH_UUID     "e8680201-9c4b-11e4-b5f7-0002a5d5c51b"
#define GLREMOTE_AUTH_UUID     {0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0xf7, 0xb5, 0xe4, 0x11, 0x4b, 0x9c, 0x01, 0x02, 0x68, 0xe8}
// GLREMOTE_TIME_UUID     "e8680203-9c4b-11e4-b5f7-0002a5d5c51b"
#define GLREMOTE_TIME_UUID     {0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0xf7, 0xb5, 0xe4, 0x11, 0x4b, 0x9c, 0x03, 0x02, 0x68, 0xe8}
// GLREMOTE_TXCHAR_UUID   "e8680103-9c4b-11e4-b5f7-0002a5d5c51b"
#define GLREMOTE_TXCHAR_UUID   {0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0xf7, 0xb5, 0xe4, 0x11, 0x4b, 0x9c, 0x03, 0x01, 0x68, 0xe8}
// GLREMOTE_RXCHAR_UUID   "e8680102-9c4b-11e4-b5f7-0002a5d5c51b"
#define GLREMOTE_RXCHAR_UUID   {0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0xf7, 0xb5, 0xe4, 0x11, 0x4b, 0x9c, 0x02, 0x01, 0x68, 0xe8}

//AM43
// AMREMOTE_SERVICE_UUID  "fe50"
#define AMREMOTE_SERVICE_UUID 0xfe50
// AMREMOTE_CHAR_UUID  "fe51"
#define AMREMOTE_CHAR_UUID 0xfe51
// AMTREMOTE_SERVICE_UUID  "1910"
#define AMTREMOTE_SERVICE_UUID 0x1910
// AMTREMOTE_RXCHAR_UUID  "2b10"
#define AMTREMOTE_RXCHAR_UUID 0x2b10
// AMTREMOTE_TXCHAR_UUID  "2b11"
#define AMTREMOTE_TXCHAR_UUID 0x2b11
//ld2410
// LDREMOTE_SERVICE_UUID  "fff0"
#define LDREMOTE_SERVICE_UUID 0xfff0
// LDREMOTE_RXCHAR_UUID  "fff1"
#define LDREMOTE_RXCHAR_UUID 0xfff1
// LDREMOTE_TXCHAR_UUID  "fff2"
#define LDREMOTE_TXCHAR_UUID 0xfff2

//delonghi
// DLREMOTE_SERVICE_UUID "00035b03-58e6-07dd-021a-08123a000300"
#define DLREMOTE_SERVICE_UUID {0x00, 0x03, 0x00, 0x3a, 0x12, 0x08, 0x1a, 0x02, 0xdd, 0x07, 0xe6, 0x58, 0x03, 0x5b, 0x03, 0x00}
// DLREMOTE_CHAR_UUID  "00035b03-58e6-07dd-021a-08123a000301"
#define DLREMOTE_CHAR_UUID {0x01, 0x03, 0x00, 0x3a, 0x12, 0x08, 0x1a, 0x02, 0xdd, 0x07, 0xe6, 0x58, 0x03, 0x5b, 0x03, 0x00}

#define PROFILE_NUM      5
#define PROFILE_A_APP_ID 0
#define PROFILE_B_APP_ID 1
#define PROFILE_C_APP_ID 2
#define PROFILE_D_APP_ID 3
#define PROFILE_E_APP_ID 4
#define INVALID_HANDLE   0
#define BLE_INPUT_BUFFSIZE 80
#define BLE_OUTPUT_BUFFSIZE 48
#define cStatus_len 160
#define otabufsize 2048
#define BleMonNum 24
#define BleMonDefTO 3000

struct BleDevSt {
int16_t  MiKettleID;
bool     btopenreq;
bool     btopen;
bool     btauthoriz;
bool     get_server;
uint8_t  notifyData[BLE_INPUT_BUFFSIZE];
int8_t   notifyDataLen;
uint8_t  readData[BLE_INPUT_BUFFSIZE];
int8_t   readDataLen;
int8_t   readDataHandle;
uint8_t  sendData[BLE_OUTPUT_BUFFSIZE];
int8_t   sendDataLen;
int8_t   sendDataHandle;
uint8_t  DEV_TYP; 
char     REQ_NAME[20];
char     DEV_NAME[20];
char     tBLEAddr[16];
uint32_t NumConn;
uint32_t PassKey;
uint8_t  LstCmd;
uint8_t  r4scounter;
uint8_t  r4sConnErr;
uint8_t  r4sAuthCount;
uint8_t  xbtauth;
uint8_t  xshedcom;
uint8_t  r4slpcom;
uint8_t  r4sppcom;
uint8_t  r4slppar1;
uint8_t  r4slppar2;
uint8_t  r4slppar3;
uint8_t  r4slppar4;
uint8_t  r4slppar5;
uint8_t  r4slppar6;
uint8_t  r4slppar7;
uint8_t  r4slppar8;
uint8_t  r4slpres;
uint8_t  t_ppcon;
uint16_t t_rspdel;
uint8_t  t_rspcnt;
uint8_t  f_Sync;
int      iRssi;
//
char     sVer[16];
char     cStatus[cStatus_len];
uint8_t  bState;
uint8_t  bHeat;
uint8_t  bLock;
uint8_t  bProg;
uint8_t  bModProg;
uint8_t  bPHour;
uint8_t  bPMin;
uint8_t  bCHour;
uint8_t  bCMin;
uint8_t  bDHour;
uint8_t  bDMin;
uint8_t  bStNl;
uint8_t  bStBl;
uint8_t  bStBp;
uint8_t  bCtemp;
uint8_t  bHtemp;
uint8_t  bAwarm;
int8_t   bBlTime;
uint8_t  RgbR;
uint8_t  RgbG;
uint8_t  RgbB;
uint32_t bSEnergy;
uint32_t bSTime;
uint32_t bSCount;
uint32_t bSHum;
uint32_t bS1Energy;
uint32_t bS1Count;
uint32_t bS2Count;
uint32_t bS3Count;
uint32_t bS4Count;
uint8_t  bCVol;
uint8_t  bCVoll;
uint8_t  bC1temp;
uint8_t  bCStemp;
uint8_t  bLtemp;
uint8_t  bKeep;
uint8_t  bEfficiency;

int      iprevRssi;
char     cprevStatus[cStatus_len];
uint8_t  bprevState;
uint8_t  bprevHeat;
uint8_t  bprevLock;
uint8_t  bprevProg;
uint8_t  bprevModProg;
uint8_t  bprevPHour;
uint8_t  bprevPMin;
uint8_t  bprevCHour;
uint8_t  bprevCMin;
uint8_t  bprevDHour;
uint8_t  bprevDMin;
uint8_t  bprevStNl;
uint8_t  bprevStBl;
uint8_t  bprevStBp;
uint8_t  bprevCtemp;
uint8_t  bprevHtemp;
uint8_t  bprevAwarm;
int8_t   bprevBlTime;
uint8_t  PRgbR;
uint8_t  PRgbG;
uint8_t  PRgbB;
uint32_t bprevSEnergy;
uint32_t bprevSTime;
uint32_t bprevSCount;
uint32_t bprevS1Count;
uint32_t bprevS2Count;
uint32_t bprevS3Count;
uint32_t bprevS4Count;
uint32_t bprevSHum;
uint32_t bprevS1Energy;
uint8_t  bprevCVol;
uint8_t  bprevCVoll;

};
struct BleMonRec {
uint16_t sto;
uint8_t  mac[32];
uint8_t  id;
uint8_t  res;
};

struct BleMonExt {
uint16_t ttick;
uint16_t mto;
char     name[20];
uint8_t  advdat[32];
uint8_t  scrsp[32];
uint8_t  advdatlen;
uint8_t  scrsplen;
uint8_t  state;
uint8_t  prstate;
int8_t   rssi;
int8_t   prrssi;
int16_t  par1;
int16_t  par2;
int16_t  par3;
int16_t  par4;
int16_t  par5;
int16_t  par6;
int16_t  par7;
int16_t  ppar1;
int16_t  ppar2;
int16_t  ppar3;
int16_t  ppar4;
int16_t  ppar5;
int16_t  ppar6;
int16_t  ppar7;
int8_t   cmrssi;
uint8_t  gtnum;
uint8_t  gttmo;
};

struct SnPari2c {
uint16_t  par1;
uint16_t  par2;
uint16_t  par3;
uint32_t  par4;
//uint16_t  par5;
//uint16_t  par6;
//uint16_t  par7;
//uint16_t  par8;
uint16_t  ppar1;
uint16_t  ppar2;
uint16_t  ppar3;
uint16_t  ppar4;
//uint16_t  ppar5;
//uint16_t  ppar6;
//uint16_t  ppar7;
//uint16_t  ppar8;
};

static uint16_t wf_retry_cnt;
static uint8_t wf_bits;
static uint8_t wf_mod;
static bool Isscanning = false;
static bool IsPassiveScan = false;
static bool StartStopScanReq = false;
static bool SetScanReq = false;
static bool f_scanproc = false;
static uint8_t hwtdiv = 0;

static esp_gattc_char_elem_t *char_elem_result   = NULL;
static esp_gattc_descr_elem_t *descr_elem_result = NULL;


esp_bd_addr_t scan_rst_bda;
esp_ble_addr_type_t scan_rst_bda_type;
int FND_RSSI = 0;
uint32_t NumWfConn;
uint32_t NumMqConn;
uint32_t MemErr;

uint16_t mqtt_port  = 1883;

esp_mqtt_client_handle_t mqttclient; 

//*** data ***
char strOFF[8];
char strON[8];
uint8_t binblemac [8];
uint8_t binwfmac [8];
char MQTT_BASE_TOPIC[8];                    // r4s
char FND_NAME[20];                          // last founded device name
char FND_ADDR[16];                          // last founded device addr
char FND_ADDRx[32];                         // last founded device addr incl :
char MQTT_USER[16];                         // MQTT Server user
char MQTT_PASSWORD[20];                     // MQTT Server password
char MQTT_SERVER[65];                       // MQTT Server
char WIFI_SSID[33];                         // network SSID for ESP32 to connect to
char WIFI_PASSWORD[65];                     // password for the network above
char NTP_SERVER[33];                        // NTP Server

uint16_t t_ppcons = 0;

#ifdef USE_TFT
//tft
uint16_t xoffs  = 0;
uint16_t MyJPGbuflen1  = 32768;
uint16_t MyJPGbuflen2  = 32768;
uint16_t MyJPGbuflen3  = 32768;
uint16_t MyJPGbuflen4  = 32768;
uint8_t MyJPGbufadj  = 0;
uint16_t jpg_time  = 32;
uint8_t tft_conn = 0;
uint8_t tft_conf = 0;
uint8_t tft_flip = 0; 
uint8_t  MyHttpMqtt = 0;
int32_t MyJPGbufidx = 0;
uint32_t JpgLoad = 0;
uint32_t JpgLoadErr = 0;
char MyHttpUri1[390];
char MyHttpUri2[390];
char MyHttpUri3[390];
char MyHttpUri4[390];
uint8_t MyHttpUridx = 0;
char *MyJPGbuf;

char MQTT_TOPP1[33];
char MQTT_TOPP2[33];
char MQTT_TOPP3[33];
char MQTT_TOPP4[33];
char MQTT_TOPP5[33];
char MQTT_TOPP6[33];
char MQTT_TOPP7[33];
char MQTT_VALP1[16];
char MQTT_VALP2[16];
char MQTT_VALP3[16];
char MQTT_VALP4[16];
char MQTT_VALP5[16];
char MQTT_VALP6[16];
char MQTT_VALP7[16];
#endif




char tESP32Addr[16];                        //esp mac
char tESP32Addr1[32];                       //esp mac

char AUTH_BASIC[51];
char bufcert[2000];
uint16_t bcertofs;
uint16_t bcertsz;
uint8_t R4SNUM = 0;                         //r4s number
uint8_t R4SNUMO = 0;                        //r4s number
uint8_t r4sppcoms = 0;
uint16_t bStateS = 0;
uint16_t bprevStateS = 0;

uint8_t bResetReason = 0;
uint8_t t_lasts = 0;
uint8_t  t_clock = 0;
uint16_t t_jpg = 0;
uint8_t  t_tinc = 0;

uint8_t fmut = 0; 
uint8_t fdebug = 0;
uint8_t foffln = 0;
uint8_t fmssl = 0; 
uint8_t fmsslbundle = 0; 
uint8_t fmsslhost = 0; 
uint8_t fmwss = 0; 
uint8_t mqtdel = 0;
uint8_t macauth = 0;
uint8_t volperc = 0;
uint8_t fkpmd = 0;
uint8_t ftvoc = 0;
uint8_t fstmirtc = 0;
uint8_t fbhlwh = 0; 
uint8_t floop = 0;
uint32_t OtaBytes;

bool mqttConnected = false;

int   iRssiESP = 0;
int   iprevRssiESP = 0;

uint8_t FDHass = 0; 
uint8_t fcommtp = 0; 
uint8_t ftrufal = 0; 
uint8_t ble_mon = 0; 
uint8_t ble_mon_refr = 0; 

uint8_t  TimeZone = 0;

uint8_t bgpio1 = 0;
uint8_t bgpio2 = 0;
uint8_t bgpio3 = 0;
uint8_t bgpio4 = 0;
uint8_t bgpio5 = 0;
uint8_t lvgpio1 = 0;
uint8_t lvgpio2 = 0;
uint8_t lvgpio3 = 0;
uint8_t lvgpio4 = 0;
uint8_t lvgpio5 = 0;
uint8_t fgpio1 = 0;
uint8_t fgpio2 = 0;
uint8_t fgpio3 = 0;
uint8_t fgpio4 = 0;
uint8_t fgpio5 = 0;
int  cntgpio1 = 0;
int  cntgpio2 = 0;
int  cntgpio3 = 0;
int  cntgpio4 = 0;
int  cntgpio5 = 0;
uint8_t lvout1 = 0;
uint8_t lvout2 = 0;
uint8_t lvout3 = 0;
uint8_t lvout4 = 0;
uint8_t lvout5 = 0;

uint32_t bStatHx6 = 0;
uint32_t bZeroHx6 = 0;
uint32_t bDivHx6 = 0;
uint32_t bprevStatHx6 = 0;
uint8_t  bgpio6 = 0;
uint16_t bprevStatG6 = 0;
uint16_t bStatG6 = 0;
uint16_t bprevStatG6h = 0;
uint16_t bStatG6h = 0;
uint8_t  bgpio7 = 0;
uint16_t bprevStatG7 = 0;
uint16_t bStatG7 = 0;
uint16_t bprevStatG7h = 0;
uint16_t bStatG7h = 0;
uint8_t  bgpio8 = 0;
uint16_t bprevStatG8 = 0;
uint16_t bStatG8 = 0;
uint16_t bprevStatG8h = 0;
uint16_t bStatG8h = 0;
uint8_t  bgpio9 = 0;
uint8_t  bgpio10 = 0;
uint8_t  f_rmds = 0;
uint32_t s_i2cdev = 0;
uint32_t f_i2cdev = 0;
uint32_t i2c_errcnt = 0;
uint8_t  i2cdevnum = 0;
uint8_t  i2cdevnumo = 0;
uint8_t  RmtDsNum = 0;
uint8_t  pwr_batmode = 0;
uint8_t  pwr_batprevmode = 0;    //mqtt
uint8_t  pwr_batpscrmode = 0;    //screen
uint8_t  pwr_batlevp = 0;
uint8_t  pwr_batprevlevp = 0;
uint16_t pwr_batlevv = 0;
uint16_t pwr_batprevlevv = 0;
uint16_t pwr_batlevc = 0;
uint16_t pwr_batprevlevc = 0;
uint16_t i2c58bltvoc = 0;
uint16_t i2c58blco2 = 0;

RingbufHandle_t RmtRgHd0 = 0;
RingbufHandle_t RmtRgHd1 = 0;
RingbufHandle_t RmtRgHd2 = 0;

const uint8_t i2c_addr[28] = {0x76,0x77,0x44,0x45,0x38,0x40,0x68,0x51,0xff,0x58,0x62,0x59,0xff,0xff,0xff,0xff,
			      0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
const uint8_t i2c_bits[28] = {0x0f,0x0f,0x03,0x03,0x03,0x03,0x01,0x00,0x00,0xc0,0x83,0x40,0x00,0x00,0x00,0x00,
			      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

bool f_update = false;


const char cssDatasheet[] = ""     // CSS
                        ""  // some CSS from ESP Easy Mega
                        "<style>* {font-family: sans-serif; font-size: 12pt; margin: 0px; padding: 0px; box-sizing: border-box; }"
                        "h1 {font-size: 16pt; color: #07D; margin: 8px 0; font-weight: bold; }"
                        "h2 {font-size: 11pt; margin: 0px; padding: 0px; font-weight: normal;}"
                        "h3 {font-size: 12pt; margin: 16px 0px 0px 0px; padding: 4px; background-color: #EEE; color: #444; font-weight: bold; }"
                        "input, select, textarea {margin: 4px; padding: 4px 8px; border-radius: 4px; background-color: #eee; border-style: solid; border-width: 1px; border-color: gray;}"
			"input:hover {background-color: #ccc; }input.wide {max-width: 500px; width:80%; }input.widenumber {max-width: 500px; width:100px; }#selectwidth {max-width: 500px; width:80%; padding: 4px 8px;}select:hover {background-color: #ccc; }.container {display: block; padding-left: 35px; margin-left: 4px; margin-top: 0px; position: relative; cursor: pointer; font-size: 12pt; -webkit-user-select: none; -moz-user-select: none; -ms-user-select: none; user-select: none; }.container input {position: absolute; opacity: 0; cursor: pointer;  }.checkmark {position: absolute; top: 0; left: 0; height: 25px;  width: 25px;  background-color: #eee; border-style: solid; border-width: 1px; border-color: gray;  border-radius: 4px;}.container:hover input ~ .checkmark {background-color: #ccc; }.container input:checked ~ .checkmark { background-color: #07D; }.checkmark:after {content: ''; position: absolute; display: none; }.container input:checked ~ .checkmark:after {display: block; }.container .checkmark:after {left: 7px; top: 3px; width: 5px; height: 10px; border: solid white; border-width: 0 3px 3px 0; -webkit-transform: rotate(45deg); -ms-transform: rotate(45deg); transform: rotate(45deg); }.container2 {display: block; padding-left: 35px; margin-left: 9px; margin-bottom: 20px; position: relative; cursor: pointer; font-size: 12pt; -webkit-user-select: none; -moz-user-select: none; -ms-user-select: none; user-select: none; }.container2 input {position: absolute; opacity: 0; cursor: pointer;  }.dotmark {position: absolute; top: 0; left: 0; height: 26px;  width: 26px;  background-color: #eee; border-style: solid; border-width: 1px; border-color: gray; border-radius: 50%;}.container2:hover input ~ .dotmark {background-color: #ccc; }.container2 input:checked ~ .dotmark { background-color: #07D;}.dotmark:after {content: ''; position: absolute; display: none; }.container2 input:checked ~ .dotmark:after {display: block; }.container2 .dotmark:after {top: 8px; left: 8px; width: 8px; height: 8px; border-radius: 50%; background: white; }#toastmessage {visibility: hidden; min-width: 250px; margin-left: -125px; background-color: #07D;color: #fff;  text-align: center;  border-radius: 4px;  padding: 16px;  position: fixed;z-index: 1; left: 282px; bottom: 30%;  font-size: 17px;  border-style: solid; border-width: 1px; border-color: gray;}#toastmessage.show {visibility: visible; -webkit-animation: fadein 0.5s, fadeout 0.5s 2.5s; animation: fadein 0.5s, fadeout 0.5s 2.5s; }@-webkit-keyframes fadein {from {bottom: 20%; opacity: 0;} to {bottom: 30%; opacity: 0.9;} }@keyframes fadein {from {bottom: 20%; opacity: 0;} to {bottom: 30%; opacity: 0.9;} }@-webkit-keyframes fadeout {from {bottom: 30%; opacity: 0.9;} to {bottom: 0; opacity: 0;} }@keyframes fadeout {from {bottom: 30%; opacity: 0.9;} to {bottom: 0; opacity: 0;} }.level_0 { color: #F1F1F1; }.level_1 { color: #FCFF95; }.level_2 { color: #9DCEFE; }.level_3 { color: #A4FC79; }.level_4 { color: #F2AB39; }.level_9 { color: #FF5500; }.logviewer {  color: #F1F1F1; background-color: #272727;  font-family: 'Lucida Console', Monaco, monospace;  height:  530px; max-width: 1000px; width: 80%; padding: 4px 8px;  overflow: auto;   border-style: solid; border-color: gray; }textarea {max-width: 1000px; width:80%; padding: 4px 8px; font-family: 'Lucida Console', Monaco, monospace; }textarea:hover {background-color: #ccc; } "
                        "table.normal th {padding: 6px; background-color: #EEE; color: #444; border-color: #888; font-weight: bold; }table.normal "
                        "td {padding: 4px; height: 30px;}table.normal "
                        "tr {padding: 4px; }table.normal {color: #000; width: 100%; min-width: 420px; border-collapse: collapse; } "
                        "td {background-color: #F8F8F8; }td.xbg {background-color: #F4F4F4; }.note {color: #444; font-style: italic; }.headermenu "
                        "{position: fixed; top: 0; left: 0; right: 0; height: 90px; padding: 8px 12px; background-color: #F8F8F8; border-bottom: 1px solid #DDD; z-index: 1;}.apheader "
                        "{padding: 8px 12px; background-color: #F8F8F8;}.bodymenu {margin-top: 96px;}.menubar "
                        "{position: inherit; top: 55px; }.menu "
                        "{float: left; padding: 4px 16px 8px 16px; color: #444; white-space: nowrap; border: solid transparent; border-width: 4px 1px 1px; border-radius: 4px 4px 0 0; text-decoration: none; }.menu.active "
                        "{color: #000; background-color: #FFF; border-color: #07D #DDD #FFF; }.menu:hover "
                        "{color: #000; background: #DEF; }.menu_button "
                        "section{overflow-x: auto; width: 100%; }@media screen and (max-width: 960px) {span.showmenulabel { display: none; }.menu { max-width: 11vw; max-width: 48px; }"
//                        ".blbox {display: inline-block;} "
			"</style>";
//update form
const char* serverIndex = "<form method='POST' action='/updating' enctype='multipart/form-data'><input type='file' name='update'><input type='submit' value='Update'></form>";
//favicon
const char imgfavicon[4286UL + 1] = {
  0x00, 0x00, 0x01, 0x00, 0x01, 0x00, 0x20, 0x20, 0x00, 0x00, 0x01, 0x00, 0x20, 0x00, 0xA8, 0x10, 0x00, 0x00, 0x16, 0x00, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x01, 0x00, 0x20, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x80, 0x10, 0x00, 0x00, 0xC3, 0x0E, 0x00, 0x00, 0xC3, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x0B, 0x0B, 0x0B, 0x1E, 0x1C, 0x17, 0x12, 0x31, 0x1D, 0x18, 0x12, 0x32, 0x0D, 0x0D, 0x0C, 0x21, 0x00, 0x00, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x04,
  0x04, 0x10, 0x51, 0x3B, 0x2A, 0x6B, 0x9E, 0x6C, 0x49, 0xB9, 0xCA, 0x85, 0x56, 0xE5, 0xE4, 0x92, 0x5A, 0xFE, 0xF0, 0x98, 0x5C, 0xFF, 0xF1, 0x98, 0x5D, 0xFF, 0xE6, 0x93, 0x5B, 0xFF, 0xCD, 0x86, 0x56, 0xE8, 0xA4, 0x6F, 0x4A, 0xBF, 0x5C, 0x42,
  0x2F, 0x77, 0x08, 0x08, 0x07, 0x17, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03,
  0x04, 0x0C, 0x7A, 0x55, 0x3B, 0x9A, 0xEB, 0x98, 0x62, 0xFF, 0xFF, 0xAA, 0x62, 0xFF, 0xFF, 0x9E, 0x4F, 0xFF, 0xFF, 0x8D, 0x40, 0xFF, 0xF7, 0x81, 0x34, 0xFF, 0xF0, 0x79, 0x2A, 0xFF, 0xF1, 0x7E, 0x33, 0xFF, 0xF6, 0x82, 0x36, 0xFF, 0xFF, 0x8B,
  0x3E, 0xFF, 0xFF, 0x9B, 0x4D, 0xFF, 0xFF, 0xAA, 0x60, 0xFF, 0xF1, 0x9C, 0x64, 0xFF, 0x8A, 0x60, 0x41, 0xAB, 0x0A, 0x09, 0x07, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x27, 0x1E, 0x17, 0x40, 0xE1, 0x98, 0x65, 0xF7, 0xFF, 0xB1, 0x65, 0xFF, 0xFA, 0x88, 0x3B, 0xFF, 0xE7, 0x73, 0x28, 0xFF, 0xE5, 0x71, 0x24, 0xFF, 0xE6, 0x72, 0x25, 0xFF, 0xE8, 0x7B, 0x33, 0xFF, 0xE6, 0x6E, 0x20, 0xFF, 0xE4, 0x64,
  0x10, 0xFF, 0xE8, 0x78, 0x2E, 0xFF, 0xE6, 0x73, 0x27, 0xFF, 0xE5, 0x71, 0x24, 0xFF, 0xE6, 0x72, 0x27, 0xFF, 0xF8, 0x84, 0x38, 0xFF, 0xFF, 0xAD, 0x61, 0xFF, 0xF1, 0xA3, 0x6A, 0xFF, 0x38, 0x29, 0x1E, 0x54, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x38, 0x28, 0x6A, 0xFF, 0xB1, 0x71, 0xFF, 0xFF, 0x99, 0x4F, 0xFF, 0xE5, 0x71, 0x26, 0xFF, 0xE5, 0x72, 0x25, 0xFF, 0xE7, 0x77, 0x2C, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE6, 0x73, 0x28, 0xFF, 0xE7, 0x79,
  0x30, 0xFF, 0xF9, 0xDB, 0xC6, 0xFF, 0xEB, 0x8A, 0x4A, 0xFF, 0xE3, 0x5E, 0x07, 0xFF, 0xE8, 0x7A, 0x31, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x72, 0x26, 0xFF, 0xE4, 0x70, 0x24, 0xFF, 0xFE, 0x93, 0x48, 0xFF, 0xFF, 0xB4,
  0x71, 0xFF, 0x68, 0x48, 0x33, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4F, 0x39, 0x29, 0x64, 0xFF, 0xB5, 0x70, 0xFF, 0xF9, 0x89, 0x3E, 0xFF, 0xE3, 0x6F, 0x22, 0xFF, 0xE7, 0x76, 0x2B, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE6, 0x74, 0x28, 0xFF, 0xE5, 0x6E, 0x1E, 0xFF, 0xFC, 0xEF, 0xE5, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xE9, 0x82, 0x3F, 0xFF, 0xE3, 0x5D, 0x04, 0xFF, 0xE8, 0x7A, 0x31, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x76,
  0x2C, 0xFF, 0xE3, 0x6F, 0x22, 0xFF, 0xF4, 0x83, 0x37, 0xFF, 0xFF, 0xB8, 0x70, 0xFF, 0x6A, 0x4B, 0x35, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x24, 0x1B, 0x42, 0xFF, 0xAD, 0x6D, 0xFF, 0xFA, 0x89, 0x3D, 0xFF, 0xE3, 0x6F, 0x22, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x74, 0x28, 0xFF, 0xE5, 0x6D, 0x1E, 0xFF, 0xFB, 0xE6, 0xD8, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFF, 0xEA, 0x85, 0x42, 0xFF, 0xE2, 0x5B, 0x02, 0xFF, 0xE8, 0x7A,
  0x31, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE4, 0x70, 0x23, 0xFF, 0xF3, 0x81, 0x35, 0xFF, 0xFF, 0xB3, 0x6F, 0xFF, 0x45, 0x33, 0x25, 0x57, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xD9, 0x97, 0x64, 0xE2, 0xFF, 0x9C, 0x50, 0xFF, 0xE3, 0x6E, 0x21, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x76, 0x2C, 0xFF, 0xE6, 0x71, 0x23, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x74, 0x28, 0xFF, 0xE5, 0x6D, 0x1E, 0xFF, 0xFA, 0xE5, 0xD7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFD,
  0xFC, 0xFF, 0xEA, 0x86, 0x44, 0xFF, 0xE3, 0x5C, 0x03, 0xFF, 0xE7, 0x79, 0x31, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE4, 0x6F, 0x22, 0xFF, 0xFF, 0x91, 0x46, 0xFF, 0xEE, 0xA3, 0x6A, 0xF8, 0x0C, 0x0A,
  0x09, 0x16, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x6F, 0x50, 0x37, 0x76, 0xFF, 0xB1, 0x68, 0xFF, 0xE5, 0x73, 0x28, 0xFF, 0xE7, 0x75,
  0x2A, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x78, 0x2E, 0xFF, 0xE5, 0x6A, 0x19, 0xFF, 0xE6, 0x70, 0x23, 0xFF, 0xE5, 0x6A, 0x1A, 0xFF, 0xE7, 0x77, 0x2C, 0xFF, 0xE5, 0x6D, 0x1E, 0xFF, 0xFB, 0xE9, 0xDD, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFB, 0xE5, 0xD8, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xEB, 0x8B, 0x4B, 0xFF, 0xE2, 0x5A, 0x01, 0xFF, 0xE7, 0x79, 0x30, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x76, 0x2C, 0xFF, 0xE4, 0x71,
  0x26, 0xFF, 0xFF, 0xAD, 0x63, 0xFF, 0x8B, 0x62, 0x43, 0x93, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xD9, 0x94, 0x60, 0xE3, 0xFF, 0x8C,
  0x3F, 0xFF, 0xE5, 0x71, 0x24, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x75, 0x2A, 0xFF, 0xE5, 0x6D, 0x1E, 0xFF, 0xF2, 0xB4, 0x8B, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xEA, 0x88, 0x48, 0xFF, 0xE2, 0x59, 0x00, 0xFF, 0xE6, 0x70,
  0x22, 0xFF, 0xFC, 0xEE, 0xE4, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xE7, 0x79, 0x30, 0xFF, 0xF9, 0xDF, 0xCD, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFE, 0xFF, 0xEE, 0x9F, 0x6A, 0xFF, 0xE5, 0x6E, 0x1F, 0xFF, 0xE7, 0x76, 0x2B, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE6, 0x72, 0x26, 0xFF, 0xF7, 0x83, 0x38, 0xFF, 0xEF, 0x9F, 0x65, 0xFA, 0x10, 0x10, 0x0E, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x47, 0x36,
  0x27, 0x48, 0xFF, 0xAA, 0x64, 0xFF, 0xE8, 0x74, 0x29, 0xFF, 0xE7, 0x76, 0x2C, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x74, 0x28, 0xFF, 0xE5, 0x6B, 0x1B, 0xFF, 0xF9, 0xDA, 0xC5, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE,
  0xFD, 0xFF, 0xE9, 0x81, 0x3C, 0xFF, 0xE1, 0x53, 0x00, 0xFF, 0xFC, 0xEF, 0xE7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xE3, 0x5E, 0x0D, 0xFF, 0xE5, 0x6B, 0x24, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF9, 0xE0, 0xD0, 0xFF, 0xE5, 0x6E,
  0x1F, 0xFF, 0xE6, 0x73, 0x26, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE5, 0x71, 0x25, 0xFF, 0xFF, 0xAA, 0x60, 0xFF, 0x63, 0x48, 0x33, 0x64, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x85, 0x5E, 0x42, 0x88, 0xFF, 0xA3, 0x56, 0xFF, 0xE5, 0x71, 0x24, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x78, 0x2E, 0xFF, 0xE5, 0x6E, 0x1F, 0xFF, 0xE5, 0x6A,
  0x19, 0xFF, 0xFA, 0xE1, 0xD1, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xF8, 0xF3, 0xFF, 0xE6, 0x73, 0x27, 0xFF, 0xF8, 0xDA, 0xC7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xE7, 0x74, 0x28, 0xFF, 0xF8, 0xD4, 0xBC, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFD, 0xF4,
  0xEE, 0xFF, 0xE8, 0x7D, 0x37, 0xFF, 0xE5, 0x6A, 0x1A, 0xFF, 0xE7, 0x78, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x71, 0x24, 0xFF, 0xFF, 0x9B, 0x4D, 0xFF, 0xA0, 0x6F, 0x4C, 0xA5, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0F, 0x14, 0x14, 0x08, 0xB3, 0x7C, 0x53, 0xBA, 0xFF, 0x92, 0x44, 0xFF, 0xE6, 0x72, 0x26, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x7A, 0x31, 0xFF, 0xE5, 0x67, 0x15, 0xFF, 0xE4, 0x67, 0x15, 0xFF, 0xFA, 0xE4, 0xD5, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFB, 0xE6, 0xD9, 0xFF, 0xFC, 0xED, 0xE3, 0xFF, 0xFE, 0xF9, 0xF6, 0xFF, 0xFA, 0xE2, 0xD2, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFE, 0xF7, 0xF3, 0xFF, 0xE7, 0x78, 0x2F, 0xFF, 0xE3, 0x60, 0x0A, 0xFF, 0xE8, 0x7A, 0x32, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x73, 0x27, 0xFF, 0xFF, 0x89,
  0x3C, 0xFF, 0xCC, 0x88, 0x58, 0xD6, 0x29, 0x28, 0x22, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x31, 0x31, 0x2A, 0x25, 0xD1, 0x8C, 0x5B, 0xDB, 0xFC, 0x86, 0x3A, 0xFF, 0xE6, 0x74, 0x28, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE8, 0x7A, 0x31, 0xFF, 0xE4, 0x67, 0x14, 0xFF, 0xE4, 0x68, 0x15, 0xFF, 0xF9, 0xDD, 0xCA, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFC, 0xF1, 0xE9, 0xFF, 0xE7, 0x78, 0x2E, 0xFF, 0xE4, 0x60, 0x0A, 0xFF, 0xE8, 0x7A, 0x32, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x75, 0x29, 0xFF, 0xF4, 0x7F, 0x34, 0xFF, 0xE3, 0x93, 0x5B, 0xF3, 0x50, 0x47, 0x38, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x43, 0x43, 0x39, 0x33, 0xDF, 0x92, 0x5C, 0xE8, 0xF6, 0x81, 0x35, 0xFF, 0xE7, 0x74,
  0x29, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x7A, 0x32, 0xFF, 0xE5, 0x6B, 0x1A, 0xFF, 0xE2, 0x59, 0x00, 0xFF, 0xF7, 0xD2,
  0xB9, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFB, 0xEB, 0xE0, 0xFF, 0xE5, 0x68, 0x15, 0xFF, 0xE4, 0x63, 0x0D, 0xFF, 0xE8, 0x7A, 0x32, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x75, 0x2A, 0xFF, 0xEF, 0x7B, 0x2F, 0xFF, 0xEE, 0x97, 0x5D, 0xFF, 0x67, 0x59, 0x46, 0x4F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x66, 0x65, 0x56, 0x31, 0xDD, 0x91,
  0x5C, 0xE7, 0xF4, 0x7F, 0x34, 0xFF, 0xE7, 0x74, 0x29, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x7A, 0x31, 0xFF, 0xE6, 0x6D,
  0x1D, 0xFF, 0xE1, 0x54, 0x00, 0xFF, 0xF6, 0xC9, 0xAD, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFA, 0xE4, 0xD6, 0xFF, 0xE3, 0x61, 0x0B, 0xFF, 0xE4, 0x65, 0x11, 0xFF, 0xE8, 0x7A, 0x32, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x75, 0x2A, 0xFF, 0xEF, 0x7A, 0x2F, 0xFF, 0xEA, 0x94, 0x5A, 0xFD, 0x83, 0x71, 0x5A, 0x4D, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x83, 0x8F, 0x7E, 0x1F, 0xD7, 0x94, 0x63, 0xD5, 0xF5, 0x7E, 0x32, 0xFF, 0xE6, 0x73, 0x27, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x7A,
  0x31, 0xFF, 0xE5, 0x69, 0x18, 0xFF, 0xE4, 0x65, 0x11, 0xFF, 0xF8, 0xD7, 0xC2, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFC, 0xEC, 0xE2, 0xFF, 0xE6, 0x72, 0x26, 0xFF, 0xE4, 0x62,
  0x0D, 0xFF, 0xE8, 0x7A, 0x32, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x74, 0x29, 0xFF, 0xF0, 0x7B, 0x30, 0xFF, 0xE2, 0x92, 0x5B, 0xED, 0x9B, 0x90,
  0x77, 0x39, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x9A, 0xA4, 0x91, 0x02, 0xD2, 0x9D, 0x70, 0xB3, 0xF6, 0x7F, 0x34, 0xFF, 0xE6, 0x72, 0x25, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x79, 0x30, 0xFF, 0xE5, 0x6B, 0x1A, 0xFF, 0xE4, 0x63, 0x0F, 0xFF, 0xFA, 0xDE, 0xCC, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFB, 0xEA, 0xDE, 0xFF, 0xFC, 0xEF, 0xE5, 0xFF, 0xFE, 0xFA, 0xF7, 0xFF, 0xFB, 0xE6, 0xD8, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFD, 0xF3, 0xED, 0xFF, 0xE7, 0x73, 0x26, 0xFF, 0xE4, 0x63, 0x0E, 0xFF, 0xE7, 0x7A, 0x32, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x73, 0x26, 0xFF, 0xF2, 0x7C,
  0x32, 0xFF, 0xDB, 0x99, 0x67, 0xCE, 0xAA, 0x9E, 0x84, 0x19, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xCA, 0xA6, 0x81, 0x81, 0xF6, 0x81, 0x39, 0xFF, 0xE6, 0x72, 0x25, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x78, 0x2E, 0xFF, 0xE6, 0x71, 0x24, 0xFF, 0xE3, 0x5F, 0x08, 0xFF, 0xF7, 0xCF, 0xB6, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xE9, 0x82, 0x3F, 0xFF, 0xF8, 0xD7, 0xC1, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xE8, 0x7E, 0x37, 0xFF, 0xFB, 0xE7, 0xDA, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFB, 0xE6, 0xD8, 0xFF, 0xE6, 0x6E, 0x1F, 0xFF, 0xE5, 0x6E, 0x1E, 0xFF, 0xE7, 0x78, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE6, 0x72, 0x25, 0xFF, 0xF5, 0x7F, 0x36, 0xFF, 0xD4, 0xA2, 0x77, 0x9E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xB6, 0x98, 0x39, 0xF1, 0x8B, 0x4B, 0xFF, 0xE8, 0x75,
  0x2A, 0xFF, 0xE7, 0x76, 0x2B, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x74, 0x28, 0xFF, 0xE5, 0x6B, 0x1B, 0xFF, 0xF8, 0xD8, 0xC2, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xEB, 0x8B, 0x4C, 0xFF, 0xE1, 0x53,
  0x00, 0xFF, 0xFC, 0xEE, 0xE5, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xE3, 0x5D, 0x0B, 0xFF, 0xE6, 0x70, 0x2A, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF9, 0xDF, 0xCD, 0xFF, 0xE5, 0x6E, 0x1F, 0xFF, 0xE6, 0x73, 0x27, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x76, 0x2C, 0xFF, 0xE6, 0x74, 0x27, 0xFF, 0xF3, 0x87, 0x43, 0xFF, 0xC9, 0xAE, 0x8B, 0x55, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0xE3, 0xA1, 0x70, 0xCF, 0xF0, 0x79, 0x30, 0xFF, 0xE6, 0x72, 0x25, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x75, 0x2A, 0xFF, 0xE5, 0x6D, 0x1C, 0xFF, 0xF3, 0xBB, 0x96, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xEB, 0x8D,
  0x4F, 0xFF, 0xE1, 0x56, 0x00, 0xFF, 0xE6, 0x6F, 0x21, 0xFF, 0xFC, 0xEE, 0xE4, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xE7, 0x76, 0x2C, 0xFF, 0xF8, 0xD6, 0xC0, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF0, 0xA7, 0x76, 0xFF, 0xE5, 0x6E,
  0x1E, 0xFF, 0xE7, 0x75, 0x2A, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x72, 0x26, 0xFF, 0xED, 0x79, 0x30, 0xFF, 0xE7, 0x9B, 0x66, 0xE8, 0xC0, 0xC3, 0xAA, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xD2, 0xB9, 0x95, 0x5E, 0xF3, 0x88, 0x46, 0xFF, 0xE7, 0x76, 0x2B, 0xFF, 0xE7, 0x75, 0x29, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x78, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE5, 0x6A,
  0x19, 0xFF, 0xE7, 0x75, 0x2A, 0xFF, 0xE5, 0x6A, 0x18, 0xFF, 0xE7, 0x76, 0x2C, 0xFF, 0xE5, 0x6D, 0x1D, 0xFF, 0xFB, 0xE9, 0xDD, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFA, 0xE2, 0xD2, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xEC, 0x93,
  0x58, 0xFF, 0xE2, 0x58, 0x00, 0xFF, 0xE7, 0x78, 0x2F, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x76, 0x2B, 0xFF, 0xE6, 0x75, 0x29, 0xFF, 0xF4, 0x84, 0x40, 0xFF, 0xD7, 0xB2, 0x8C, 0x7B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE5, 0xA3, 0x71, 0xCF, 0xF0, 0x7D, 0x36, 0xFF, 0xE5, 0x72, 0x25, 0xFF, 0xE7, 0x77, 0x2C, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x76, 0x2B, 0xFF, 0xE6, 0x6F, 0x21, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x74, 0x28, 0xFF, 0xE5, 0x6D, 0x1E, 0xFF, 0xFA, 0xE5, 0xD7, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xED, 0x9B, 0x64, 0xFF, 0xE2, 0x5A, 0x01, 0xFF, 0xE7, 0x78, 0x2F, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x71, 0x25, 0xFF, 0xEE, 0x7B, 0x34, 0xFF, 0xE9, 0x9D, 0x69, 0xE5, 0xCC, 0xC6,
  0xAA, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xCE, 0xCA, 0xAE, 0x19, 0xEB, 0x99, 0x62, 0xF3, 0xEE, 0x78,
  0x30, 0xFF, 0xE5, 0x71, 0x24, 0xFF, 0xE7, 0x76, 0x2C, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x74, 0x28, 0xFF, 0xE5, 0x6D, 0x1E, 0xFF, 0xFA, 0xE5, 0xD7, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xEB, 0x8C, 0x4D, 0xFF, 0xE2, 0x59, 0x00, 0xFF, 0xE7, 0x78, 0x2F, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE5, 0x71, 0x24, 0xFF, 0xEC, 0x77, 0x2D, 0xFF, 0xEE, 0x93,
  0x59, 0xFF, 0xD1, 0xC3, 0xA4, 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0xD3, 0xC5, 0xA6, 0x37, 0xEF, 0x91, 0x55, 0xFF, 0xED, 0x7A, 0x32, 0xFF, 0xE5, 0x73, 0x27, 0xFF, 0xE7, 0x75, 0x2A, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE6, 0x74, 0x28, 0xFF, 0xE5, 0x6D,
  0x1E, 0xFF, 0xFC, 0xED, 0xE4, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xEA, 0x89, 0x48, 0xFF, 0xE2, 0x5B, 0x02, 0xFF, 0xE7, 0x79, 0x30, 0xFF, 0xE7, 0x77, 0x2D, 0xFF, 0xE7, 0x77, 0x2E, 0xFF, 0xE7, 0x76, 0x2B, 0xFF, 0xE6, 0x72, 0x26, 0xFF, 0xEC, 0x7A,
  0x31, 0xFF, 0xF0, 0x8D, 0x4F, 0xFF, 0xD5, 0xBE, 0x9C, 0x51, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xD1, 0xC6, 0xA9, 0x43, 0xEF, 0x94, 0x5A, 0xFF, 0xF0, 0x7C, 0x35, 0xFF, 0xE7, 0x74, 0x28, 0xFF, 0xE6, 0x71, 0x24, 0xFF, 0xE7, 0x76, 0x2B, 0xFF, 0xE7, 0x77,
  0x2D, 0xFF, 0xE6, 0x73, 0x28, 0xFF, 0xE7, 0x75, 0x2A, 0xFF, 0xFB, 0xE8, 0xDB, 0xFF, 0xED, 0x97, 0x5F, 0xFF, 0xE2, 0x5B, 0x01, 0xFF, 0xE7, 0x7A, 0x31, 0xFF, 0xE7, 0x78, 0x2E, 0xFF, 0xE7, 0x76, 0x2B, 0xFF, 0xE6, 0x72, 0x25, 0xFF, 0xE6, 0x73,
  0x27, 0xFF, 0xEF, 0x7B, 0x33, 0xFF, 0xF0, 0x91, 0x55, 0xFF, 0xD4, 0xBE, 0x9D, 0x5D, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC8, 0xD4, 0xBE, 0x18, 0xE5, 0xA6, 0x76, 0xCC, 0xF2, 0x83, 0x40, 0xFF, 0xEE, 0x77,
  0x2D, 0xFF, 0xE8, 0x76, 0x2B, 0xFF, 0xE6, 0x72, 0x26, 0xFF, 0xE6, 0x71, 0x24, 0xFF, 0xE7, 0x7B, 0x32, 0xFF, 0xE8, 0x78, 0x2E, 0xFF, 0xE4, 0x62, 0x0D, 0xFF, 0xE6, 0x76, 0x2B, 0xFF, 0xE6, 0x72, 0x26, 0xFF, 0xE6, 0x72, 0x26, 0xFF, 0xE7, 0x76,
  0x2B, 0xFF, 0xEE, 0x77, 0x2D, 0xFF, 0xF3, 0x80, 0x3A, 0xFF, 0xE7, 0xA2, 0x6F, 0xDF, 0xCE, 0xCA, 0xAF, 0x26, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0xD5, 0xC0, 0x9F, 0x56, 0xE6, 0xA4, 0x73, 0xDA, 0xEE, 0x8B, 0x4C, 0xFF, 0xF1, 0x7C, 0x34, 0xFF, 0xEF, 0x78, 0x2F, 0xFF, 0xEC, 0x76, 0x2A, 0xFF, 0xEA, 0x6E, 0x1E, 0xFF, 0xEB, 0x76, 0x2B, 0xFF, 0xEC, 0x77, 0x2D, 0xFF, 0xEE, 0x78,
  0x2E, 0xFF, 0xF0, 0x7B, 0x33, 0xFF, 0xEF, 0x89, 0x48, 0xFF, 0xE7, 0xA1, 0x6E, 0xE3, 0xD6, 0xBE, 0x9B, 0x66, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xCF, 0xC8, 0xAB, 0x2C, 0xDA, 0xB8, 0x92, 0x81, 0xE1, 0xAB, 0x7E, 0xB4, 0xE6, 0xA1, 0x6E, 0xD5, 0xE7, 0x9C, 0x68, 0xEC, 0xE7, 0x9D,
  0x67, 0xED, 0xE6, 0xA0, 0x6C, 0xD8, 0xE1, 0xAA, 0x7C, 0xB8, 0xDB, 0xB6, 0x90, 0x88, 0xD3, 0xC3, 0xA6, 0x37, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC9, 0xD1, 0xBB, 0x08, 0xD0, 0xC7, 0xA9, 0x35, 0xD4, 0xBE,
  0x9C, 0x59, 0xD6, 0xC0, 0xA0, 0x74, 0xD7, 0xC0, 0xA1, 0x76, 0xD5, 0xBD, 0x9B, 0x5E, 0xD1, 0xC4, 0xA5, 0x39, 0xCA, 0xCF, 0xB7, 0x0B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xF8, 0x1F, 0xFF, 0xFF, 0xC0, 0x03, 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0xFE, 0x00, 0x00, 0x7F, 0xFC, 0x00, 0x00, 0x3F, 0xF8, 0x00, 0x00, 0x1F, 0xF0, 0x00, 0x00, 0x0F, 0xF0, 0x00, 0x00, 0x07, 0xE0, 0x00, 0x00, 0x07, 0xE0, 0x00,
  0x00, 0x03, 0xC0, 0x00, 0x00, 0x03, 0xC0, 0x00, 0x00, 0x03, 0x80, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x01, 0xC0, 0x00, 0x00, 0x03, 0xC0, 0x00,
  0x00, 0x03, 0xE0, 0x00, 0x00, 0x03, 0xE0, 0x00, 0x00, 0x07, 0xF0, 0x00, 0x00, 0x07, 0xF0, 0x00, 0x00, 0x0F, 0xF8, 0x00, 0x00, 0x1F, 0xFC, 0x00, 0x00, 0x3F, 0xFE, 0x00, 0x00, 0x7F, 0xFF, 0x80, 0x01, 0xFF, 0xFF, 0xE0, 0x07, 0xFF, 0xFF, 0xF0,
  0x0F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00
};
